import { __decorate } from 'tslib';
import { ɵɵdefineInjectable, Injectable, Input, Component, NgModule } from '@angular/core';
import { BehaviorSubject, Subject, race, timer, of } from 'rxjs';
import { map, debounce, take, tap, switchMap } from 'rxjs/operators';
import { BrowserModule } from '@angular/platform-browser';

var SpinnerActionType;
(function (SpinnerActionType) {
    SpinnerActionType[SpinnerActionType["Show"] = 0] = "Show";
    SpinnerActionType[SpinnerActionType["Hide"] = 1] = "Hide";
    SpinnerActionType[SpinnerActionType["None"] = 2] = "None";
})(SpinnerActionType || (SpinnerActionType = {}));
;
var SpinnerAction = /** @class */ (function () {
    function SpinnerAction() {
    }
    return SpinnerAction;
}());
var NgxLoadingSpinnerService = /** @class */ (function () {
    function NgxLoadingSpinnerService() {
        this.visibleSubject = new BehaviorSubject(false);
        this.delayedSubject = new Subject();
        this.immediateSubject = new Subject();
        this.actionRequested = SpinnerActionType.None;
        this.actionExecuted = SpinnerActionType.None;
        this._timeout = 10000;
        this._threshold = 500;
        this.raceSubject();
    }
    Object.defineProperty(NgxLoadingSpinnerService.prototype, "timeout", {
        set: function (timeout) {
            this._timeout = timeout;
            this.patchShow();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxLoadingSpinnerService.prototype, "threshold", {
        set: function (threshold) {
            this._threshold = threshold;
            this.patchShow();
        },
        enumerable: true,
        configurable: true
    });
    NgxLoadingSpinnerService.prototype.raceSubject = function () {
        var _this = this;
        var delayedSubject = this.delayedSubject.pipe(map(function (t) { return ({ type: t, immediate: false }); }));
        var immediateSubject = this.immediateSubject.pipe(map(function (t) { return ({ type: t, immediate: true }); }));
        var thresholdObserver = race(delayedSubject.pipe(debounce(function () { return timer(_this._threshold); })), immediateSubject).pipe(take(1), tap(function (action) {
            _this.updatePublicState(action.type);
        }), map(function (action) {
            if (action.type == SpinnerActionType.Show) {
                return { type: SpinnerActionType.Hide, immediate: false };
            }
            else {
                return action;
            }
        }));
        var timeoutObserver = race(thresholdObserver.pipe(switchMap(function (action) { return (action.immediate ? of(action) : timer(_this._timeout).pipe(map(function (_) { return action; }))); })), immediateSubject).pipe(take(1));
        timeoutObserver.subscribe(function (action) {
            _this.updatePublicState(action.type);
        }, function (error) { return console.error(error); }, function () { return _this.raceSubject(); });
    };
    NgxLoadingSpinnerService.prototype.show = function () {
        this.actionRequested = SpinnerActionType.Show;
        this.immediateSubject.next(SpinnerActionType.None);
        this.delayedSubject.next(SpinnerActionType.Show);
    };
    NgxLoadingSpinnerService.prototype.hide = function () {
        this.actionRequested = SpinnerActionType.Hide;
        this.immediateSubject.next(SpinnerActionType.Hide);
    };
    //
    NgxLoadingSpinnerService.prototype.patchShow = function () {
        if (this.actionRequested == SpinnerActionType.Show) {
            if (this.actionExecuted == SpinnerActionType.Show) {
            }
            else {
                this.immediateSubject.next(SpinnerActionType.None);
                this.delayedSubject.next(SpinnerActionType.Show);
            }
        }
    };
    NgxLoadingSpinnerService.prototype.updatePublicState = function (type) {
        switch (type) {
            case SpinnerActionType.Show:
                this.actionExecuted = type;
                this.visibleSubject.next(true);
                break;
            case SpinnerActionType.Hide:
                this.actionExecuted = type;
                this.visibleSubject.next(false);
                break;
        }
    };
    NgxLoadingSpinnerService.ɵprov = ɵɵdefineInjectable({ factory: function NgxLoadingSpinnerService_Factory() { return new NgxLoadingSpinnerService(); }, token: NgxLoadingSpinnerService, providedIn: "root" });
    NgxLoadingSpinnerService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], NgxLoadingSpinnerService);
    return NgxLoadingSpinnerService;
}());

var NgxLoadingSpinnerComponent = /** @class */ (function () {
    function NgxLoadingSpinnerComponent(service) {
        this.service = service;
        this.zIndex = 100;
        this.visible = false;
    }
    Object.defineProperty(NgxLoadingSpinnerComponent.prototype, "timeout", {
        set: function (v) {
            this.service.timeout = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgxLoadingSpinnerComponent.prototype, "threshold", {
        set: function (v) {
            this.service.threshold = v;
        },
        enumerable: true,
        configurable: true
    });
    NgxLoadingSpinnerComponent.prototype.ngOnInit = function () {
        this.subscribeToService();
    };
    NgxLoadingSpinnerComponent.prototype.subscribeToService = function () {
        var _this = this;
        this.service.visibleSubject.subscribe(function (spinnerVisible) {
            _this.visible = spinnerVisible;
        });
    };
    NgxLoadingSpinnerComponent.ctorParameters = function () { return [
        { type: NgxLoadingSpinnerService }
    ]; };
    __decorate([
        Input()
    ], NgxLoadingSpinnerComponent.prototype, "timeout", null);
    __decorate([
        Input()
    ], NgxLoadingSpinnerComponent.prototype, "threshold", null);
    __decorate([
        Input()
    ], NgxLoadingSpinnerComponent.prototype, "zIndex", void 0);
    __decorate([
        Input()
    ], NgxLoadingSpinnerComponent.prototype, "loadingText", void 0);
    __decorate([
        Input()
    ], NgxLoadingSpinnerComponent.prototype, "template", void 0);
    NgxLoadingSpinnerComponent = __decorate([
        Component({
            selector: 'ngx-loading-spinner',
            template: "<div class=\"overlay noselect\" [class.on]=\"visible\" [class.off]=\"!visible\" [style.zIndex]=\"zIndex\">\r\n    <div class=\"spinner-container\" *ngIf=\"!template; else showTemplate\">\r\n        <div class=\"spinner-item lds-default\"><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div><div></div></div>\r\n        <div class=\"spinner-item loading-text\" *ngIf=\"loadingText\">{{ loadingText }}</div>\r\n    </div>\r\n</div>\r\n\r\n<ng-template #showTemplate>\r\n  <div class=\"spinner-container\" [innerHTML]=\"template\"></div>\r\n</ng-template>\r\n",
            styles: [".noselect{-webkit-touch-callout:none;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}.overlay{background-color:rgba(24,24,24,.5);position:fixed;top:0;left:0;width:100%;height:100%;transition:opacity .5s;opacity:0;pointer-events:none}.overlay.on{opacity:1;pointer-events:auto}.spinner-container{color:#fff;font-family:Arial,\"Helvetica Neue\",Helvetica,sans-serif;height:100%;display:flex;flex-direction:column;justify-content:center;align-items:center}.loading-text{text-shadow:0 0 .5rem #202020;font-size:1.5em;text-align:center}.lds-default{display:inline-block;position:relative;width:80px;height:80px}.lds-default div{position:absolute;width:6px;height:6px;background:#fff;border-radius:50%;-webkit-animation:1.2s linear infinite lds-default;animation:1.2s linear infinite lds-default}.lds-default div:nth-child(1){-webkit-animation-delay:0s;animation-delay:0s;top:37px;left:66px}.lds-default div:nth-child(2){-webkit-animation-delay:-.1s;animation-delay:-.1s;top:22px;left:62px}.lds-default div:nth-child(3){-webkit-animation-delay:-.2s;animation-delay:-.2s;top:11px;left:52px}.lds-default div:nth-child(4){-webkit-animation-delay:-.3s;animation-delay:-.3s;top:7px;left:37px}.lds-default div:nth-child(5){-webkit-animation-delay:-.4s;animation-delay:-.4s;top:11px;left:22px}.lds-default div:nth-child(6){-webkit-animation-delay:-.5s;animation-delay:-.5s;top:22px;left:11px}.lds-default div:nth-child(7){-webkit-animation-delay:-.6s;animation-delay:-.6s;top:37px;left:7px}.lds-default div:nth-child(8){-webkit-animation-delay:-.7s;animation-delay:-.7s;top:52px;left:11px}.lds-default div:nth-child(9){-webkit-animation-delay:-.8s;animation-delay:-.8s;top:62px;left:22px}.lds-default div:nth-child(10){-webkit-animation-delay:-.9s;animation-delay:-.9s;top:66px;left:37px}.lds-default div:nth-child(11){-webkit-animation-delay:-1s;animation-delay:-1s;top:62px;left:52px}.lds-default div:nth-child(12){-webkit-animation-delay:-1.1s;animation-delay:-1.1s;top:52px;left:62px}@-webkit-keyframes lds-default{0%,100%,20%,80%{transform:scale(1)}50%{transform:scale(1.5)}}@keyframes lds-default{0%,100%,20%,80%{transform:scale(1)}50%{transform:scale(1.5)}}"]
        })
    ], NgxLoadingSpinnerComponent);
    return NgxLoadingSpinnerComponent;
}());

var NgxLoadingSpinnerModule = /** @class */ (function () {
    function NgxLoadingSpinnerModule() {
    }
    NgxLoadingSpinnerModule_1 = NgxLoadingSpinnerModule;
    NgxLoadingSpinnerModule.forRoot = function () {
        return {
            ngModule: NgxLoadingSpinnerModule_1,
            providers: [NgxLoadingSpinnerService]
        };
    };
    var NgxLoadingSpinnerModule_1;
    NgxLoadingSpinnerModule = NgxLoadingSpinnerModule_1 = __decorate([
        NgModule({
            declarations: [NgxLoadingSpinnerComponent],
            imports: [
                BrowserModule
            ],
            exports: [NgxLoadingSpinnerComponent]
        })
    ], NgxLoadingSpinnerModule);
    return NgxLoadingSpinnerModule;
}());

/*
 * Public API Surface of ngx-loading-spinner
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxLoadingSpinnerComponent, NgxLoadingSpinnerModule, NgxLoadingSpinnerService };
//# sourceMappingURL=k-adam-ngx-loading-spinner.js.map
